@@ -1,5 +1,7 @@
 import warnings
 import math
+from typing import Optional, List
+
 
 import numpy as np
 import pandas as pd
@@ -804,29 +806,27 @@ def ecart_type_pondere(colonne, poids):
     return math.sqrt(variance)
 
 
-import warnings
-import math
-
-import numpy as np
-import pandas as pd
-from pandas.api.types import is_numeric_dtype
-
-from scipy.stats import chi2_contingency
-from scipy.stats.distributions import chi2
-from scipy.stats import hypergeom
-from scipy.stats import norm
-
-import statsmodels.api as sm
-import statsmodels.formula.api as smf
-
-import plotly.graph_objects as go
 
 
-import statsmodels.api as sm
-from statsmodels.formula.api import ols
+def _escape_quotes(variable: str) -> str:
+    """ 
+    Complète la fonction Q() de patsy pour échapper les caractères problématiques
+    Parameters
+    ----------
+    variable : str
+        nom de variable
+        
+    Returns
+    -------
+    str
+        nom de variable modifiée
+
+    """
+    return variable.replace('"', '\\"')
 
 
-def catdes(df, vardep, varindep=False, proba = 0.05, weight=False, mod=False):
+def catdes(df: pd.DataFrame, vardep: str, varindep: List[str] = None,
+           proba: float = 0.05, weight: Optional[str] = False, mod: bool = False):
     """
     Calcule la relation entre une variable catégorielle et plusieurs variables.
     Implémentation de la fonction catdes de FactoMineR.
@@ -874,19 +874,14 @@ def catdes(df, vardep, varindep=False, proba = 0.05, weight=False, mod=False):
     if is_numeric_dtype(df[vardep]):
         print("Attention, la variable dépendante est numérique")
         return None
-    
-    # Pondération à 1 si pas de pondération
-    if not weight :
-        df["weight"] = [1]*len(df)
-        weight = "weight"
 
     # Construction de la liste de variables
     cols_num = []
     cols_cat = []
     if not varindep:
         # Cas où les variables ne sont pas proposées
-        cols_cat = [c for c in df.columns if not is_numeric_dtype(df[c]) and c != vardep]
-        cols_num = [c for c in df.columns if is_numeric_dtype(df[c])]
+        cols_cat = [c for c in df.columns if not is_numeric_dtype(df[c]) and c != vardep] # sans la variable dépendante
+        cols_num = [c for c in df.columns if is_numeric_dtype(df[c]) and c != weight] # sans la pondération
     else:
         # Cas où les variables sont proposées
         for i in varindep:
@@ -894,6 +889,11 @@ def catdes(df, vardep, varindep=False, proba = 0.05, weight=False, mod=False):
                 cols_num.append(i)
             else:
                 cols_cat.append(i)
+
+    # Pondération à 1 si pas de pondération
+    if not weight :
+        df["weight"] = [1]*len(df)
+        weight = "weight"
     
     # Calcul de l'association par variables
     
@@ -927,19 +927,13 @@ def catdes(df, vardep, varindep=False, proba = 0.05, weight=False, mod=False):
     
     tableau_num_var = []
     var_num_corr = []
-
+    
     # Pour chaque variable numérique
-    for v in cols_num:
-        
-        # Gestion des espaces dans les noms pour la formule
-        v_m = v.replace(" ","_").replace(":","")
-        vardep_m = vardep.replace(" ","_").replace(":","")
-        df[v_m] = df[v]
-        df[vardep_m] = df[vardep]
-        
-        # Calcul d'un ANOVA
-        model = ols(f"{v_m} ~ C({vardep_m})", data=df,
-                                weights=weight).fit()       
+    for v in cols_num:        
+        # Calcul d'une ANOVA
+        # Utilisation de Q() pour échapper les variables
+        #  See https://patsy.readthedocs.io/en/latest/builtins-reference.html#patsy.builtins.Q
+        model = ols(f"Q(\"{_escape_quotes(v)}\") ~ C(Q(\"{_escape_quotes(vardep)}\"))", data=df).fit()
         aov_table = sm.stats.anova_lm(model, typ=2)
         
         # Paramètre de l'association
@@ -1027,10 +1021,10 @@ def catdes(df, vardep, varindep=False, proba = 0.05, weight=False, mod=False):
     for v in var_num_corr:
 
         # Calcul de paramètres
-        moy_mod = df.groupby("sexe").apply(lambda x : moyenne_ponderee(x[v],x[weight]))
-        n_mod = df.groupby("sexe")[v].count()
+        moy_mod = df.groupby(vardep).apply(lambda x : moyenne_ponderee(x[v],x[weight]))
+        n_mod = df.groupby(vardep)[v].count()
         n = sum(n_mod)
-        sd_mod = df.groupby("sexe").apply(lambda x : ecart_type_pondere(x[v],x[weight]))
+        sd_mod = df.groupby(vardep).apply(lambda x : ecart_type_pondere(x[v],x[weight]))
         moy = moyenne_ponderee(df[v],df[weight])
         sd =  ecart_type_pondere(df[v],df[weight])
 
